/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.plugin.archetype;

import java.io.File;
import java.util.List;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.factory.ArtifactFactory;
import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.artifact.resolver.ArtifactResolver;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.bluestemsoftware.open.eoa.plugin.util.FileUtils;

/**
 * 
 * @aggregator
 * @requiresProject = false;
 * @goal archetype
 */
public class ArchetypeMojo extends AbstractMojo {

    /**
     * The base directory of the project being tested.
     * 
     * @parameter expression="${basedir}"
     * @required
     */
    private File basedir;

    /**
     * Creates the artifact
     * 
     * @component
     */
    private ArtifactFactory artifactFactory;

    /**
     * The plugin remote repositories declared in the pom.
     * 
     * @parameter expression="${project.pluginArtifactRepositories}"
     */
    private List<?> remoteRepositories;

    /**
     * Resolves the artifacts needed.
     * 
     * @component
     */
    private ArtifactResolver artifactResolver;

    /**
     * The localRepository. To obtain the directory of localRepository in unit tests use
     * System.setProperty( "localRepository").
     * 
     * @parameter expression="${localRepository}"
     * @required
     * @readonly
     */
    private ArtifactRepository localRepository;

    /**
     * Project type. Valid values are 'component', 'extension' and 'feature'.
     * 
     * @parameter expression="${type}"
     * @required
     */
    private String type;

    private static final String GROUP_ID = "org.bluestemsoftware.open.eoa.ext";

    /*
     * (non-Javadoc)
     * @see org.apache.maven.plugin.AbstractMojo#execute()
     */
    public void execute() throws MojoExecutionException {

        if (type.equals("component")) {
            try {
                createComponentProjectArchetype();
            } catch (Exception ex) {
                throw new MojoExecutionException("Error creating component project archetype. " + ex);
            }
        } else if (type.equals("extension")) {
            try {
                createExtensionProjectArchetype();
            } catch (Exception ex) {
                throw new MojoExecutionException("Error creating extension project archetype. " + ex);
            }
        } else if (type.equals("feature")) {
            try {
                createFeatureProjectArchetype();
            } catch (Exception ex) {
                throw new MojoExecutionException("Error creating feature project archetype. " + ex);
            }
        } else {
            throw new MojoExecutionException(
                    "Unrecognized type. Valid values are 'component', 'factory' and 'feature'.");
        }

    }

    public void createComponentProjectArchetype() throws Exception {
        Artifact a = null;
        a = artifactFactory.createArtifact(GROUP_ID, "component-project-archetype", "RELEASE", "compile", "jar");
        artifactResolver.resolve(a, remoteRepositories, localRepository);
        File destination = new File(basedir, "my-application/");
        destination.mkdirs();
        FileUtils.unpackArchive(a.getFile(), destination, false);
        FileUtils.deleteFile(new File(destination, "META-INF/maven/"));
        new File(destination, "META-INF/MANIFEST.MF").delete();
        createEmptyDirs(destination);
    }
    
    public void createExtensionProjectArchetype() throws Exception {
        Artifact a = null;
        a = artifactFactory.createArtifact(GROUP_ID, "extension-project-archetype", "RELEASE", "compile", "jar");
        artifactResolver.resolve(a, remoteRepositories, localRepository);
        File destination = new File(basedir, "my-extension/");
        destination.mkdirs();
        FileUtils.unpackArchive(a.getFile(), destination, false);
        FileUtils.deleteFile(new File(destination, "META-INF/maven/"));
        new File(destination, "META-INF/MANIFEST.MF").delete();
        createEmptyDirs(destination);
    }
    
    public void createFeatureProjectArchetype() throws Exception {
        Artifact a = null;
        a = artifactFactory.createArtifact(GROUP_ID, "feature-project-archetype", "RELEASE", "compile", "jar");
        artifactResolver.resolve(a, remoteRepositories, localRepository);
        File destination = new File(basedir, "my-feature/");
        destination.mkdirs();
        FileUtils.unpackArchive(a.getFile(), destination, false);
        FileUtils.deleteFile(new File(destination, "META-INF/maven/"));
        new File(destination, "META-INF/MANIFEST.MF").delete();
        createEmptyDirs(destination);
    }

    /*
     * maven archiver will not archive empty dirs. so we create the empty dirs manually
     */
    private void createEmptyDirs(File destination) {
        new File(destination, "src/main/java/").mkdirs();
        new File(destination, "src/test/java/").mkdirs();
        new File(destination, "src/test/resources/").mkdirs();
        new File(destination, "src/it/suite1/system/java/").mkdirs();
    }

}
