/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.plugin.util;

import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.factory.ArtifactFactory;
import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.artifact.resolver.ArtifactResolver;
import org.apache.maven.artifact.resolver.filter.ExcludesArtifactFilter;
import org.apache.maven.artifact.versioning.InvalidVersionSpecificationException;
import org.apache.maven.artifact.versioning.VersionRange;
import org.apache.maven.model.Dependency;
import org.apache.maven.model.Exclusion;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;

public class DependencyHelper {

    /**
     * Collects the dependency artifacts, but does not resolve them. This is important because component
     * dependencies which are not classpath artifacts may be cyclic and will break the build if we
     * blindly resolve all project dependencies.
     * 
     * @param project
     * @param artifactFactory
     * @return
     * @throws MojoExecutionException
     */
    public static Set<Artifact> getProjectDependencies(MavenProject project, ArtifactFactory artifactFactory) throws MojoExecutionException {

        Set<Artifact> projectDependencies = new HashSet<Artifact>();

        for (Object dependencyObj : project.getDependencies()) {

            Dependency dependency = (Dependency)dependencyObj;
            String groupID = dependency.getGroupId();
            String artifactID = dependency.getArtifactId();
            VersionRange versionRange = null;
            try {
                versionRange = VersionRange.createFromVersionSpec(dependency.getVersion());
            } catch (InvalidVersionSpecificationException ie) {
                throw new MojoExecutionException(ie.toString());
            }
            String scope = dependency.getScope() == null ? Artifact.SCOPE_COMPILE : dependency.getScope();
            String type = dependency.getType() == null ? "jar" : dependency.getType();
            String classifier = dependency.getClassifier();
            boolean isOptional = dependency.isOptional();

            Artifact artifact = artifactFactory.createDependencyArtifact(groupID, artifactID, versionRange, type,
                    classifier, scope, isOptional);
            projectDependencies.add(artifact);

            if (scope.equals(Artifact.SCOPE_SYSTEM)) {
                File file = new File(dependency.getSystemPath());
                artifact.setFile(file);
            }

            if (dependency.getExclusions() != null) {
                List<String> patterns = new ArrayList<String>();
                for (Object exclusionObj : dependency.getExclusions()) {
                    Exclusion exclusion = (Exclusion)exclusionObj;
                    StringBuilder sb = new StringBuilder();
                    sb.append(exclusion.getGroupId());
                    sb.append(":");
                    sb.append(exclusion.getArtifactId());
                    patterns.add(sb.toString());
                }
                ExcludesArtifactFilter excludesFilter = new ExcludesArtifactFilter(patterns);
                artifact.setDependencyFilter(excludesFilter);
            }

        }

        return projectDependencies;

    }
    
    public static void resolveArtifact(Artifact artifact, List<?> remoteRepositories, ArtifactResolver artifactResolver, ArtifactRepository localRepository) throws MojoExecutionException {

        if (artifact.isResolved()) {
            return;
        }
        
        // if artifact is a snapshot, save off the artifact version before resolving. for
        // some reason, as of version 2.1-SNAPSHOT, the DefaultArtifactResolver overwrites
        // SNAPSHOT with timestamp version format (baseVersion and version), which will
        // create a VERY obscure bug, i.e. pom will not be located and will instead be
        // stubbed, i.e. no dependencies ...

        try {
            if (artifact.isSnapshot()) {
                String version = artifact.getVersion();
                artifactResolver.resolve(artifact, remoteRepositories, localRepository);
                artifact.selectVersion(version);
            } else {
                artifactResolver.resolve(artifact, remoteRepositories, localRepository);
            }
        } catch (Exception ex) {
            throw new MojoExecutionException("Error resolving artifact. " + ex);
        } 

    }
    
}
